/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.checker.logic;

import java.util.Set;

import fr.supelec.adlv.Component;
import fr.supelec.adlv.checker.PortInstance;

public class ActivePredicate extends Predicate {
	private final boolean active;
	private final Component comp;

	public ActivePredicate(boolean active, Component comp) {
		this.active = active;
		this.comp = comp;
	}


	@Override
	public Predicate neg() {
		return new ActivePredicate(!active, comp);
	}

	@Override
	public boolean isNegationOf(Predicate p) {
		if(p instanceof ActivePredicate) {
			ActivePredicate ap = (ActivePredicate)p;
			return ap.comp == comp && ap.active != active;
		}
		return false;
	}

	@Override
	public String toString() {
		return (active ? "" : "in") + "active(" + comp.getName() + ")";
	}

	public Component getComponent() {
		return comp;
	}

	/*public String getComponentName() {
		return comp.getName();
	}*/

	public boolean isActive() {
		return active;
	}

	@Override
	public Set<PortInstance> getInputSignals() {
		// undefined for this class
		return null;
	}

}
