/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.checker.logic;

import java.util.Set;

import fr.supelec.adlv.checker.PortInstance;

public abstract class BinaryExpr extends Expr {
	protected final Expr a, b;

	protected BinaryExpr(Expr a, Expr b) {
		this.a = a;
		this.b = b;
	}

	public Expr getA() {
		return a;
	}

	public Expr getB() {
		return b;
	}

	@Override
	public boolean equals(Object o) {
		if(o instanceof BinaryExpr) {
			BinaryExpr oo = (BinaryExpr)o;
			return a.equals(oo.a) && b.equals(oo.b);
		} else return false;
	}

	/**
	 * Creates a new instance of the (concrete, final) class.
	 * @param a
	 * @param b
	 * @return
	 */
	public abstract BinaryExpr create(Expr a, Expr b);

	@Override
	public Set<PortInstance> getInputSignals() {
		Set<PortInstance> is = a.getInputSignals();
		is.addAll(b.getInputSignals());
		return is;
	}
}
