/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.checker.logic;

import java.util.Set;

import fr.supelec.adlv.checker.PortInstance;

public class ConnectedPredicate extends Predicate {
	private final boolean connected;
	private final PortInstance toPort;
	private final PortInstance fromPort;

	public ConnectedPredicate(PortInstance fromPort, PortInstance toPort, boolean connected) {
		this.fromPort = fromPort;
		this.toPort = toPort;
		this.connected = connected;
	}

	@Override
	public Predicate neg() {
		return new ConnectedPredicate(fromPort, toPort, !connected);
	}

	@Override
	public boolean isNegationOf(Predicate p) {
		if(p instanceof ConnectedPredicate) {
			ConnectedPredicate cp = (ConnectedPredicate)p;
			return cp.fromPort == fromPort && cp.toPort == toPort
					&& cp.connected != connected;
		}
		return false;
	}

	@Override
	public String toString() {
		String res = toPort + " << " + fromPort;
		if(connected) return res;
		else return "!(" + res + ")";
	}

	public PortInstance getToPort() {
		return toPort;
	}

	public PortInstance getFromPort() {
		return fromPort;
	}

	public boolean isConnected() {
		return connected;
	}

	@Override
	public Set<PortInstance> getInputSignals() {
		return null; // not implemented for this class
	}
}
