/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.generator;

import java.io.File;
import java.io.IOException;

import fr.supelec.adlv.*;

import fr.supelec.adlv.parser.EmfAdlvHelper;

public class CppGenerator extends AdlvGenerator {
	CppGenerator( EmfAdlvHelper helper ) {
		helper_ = helper;
	}
	
	void generate( String resultsPath ) throws IOException {
		
		for( Component c : helper_.getApplication().getComponents() ) {
			ComponentType t = c.getType();
			if( ! ( t instanceof InternalComponentType )) {
				generateImplFiles( t, resultsPath );
			}
		}
	}
	
	private void generateImplFiles( ComponentType component, String resultsPath ) throws IOException {
		String cname = component.getName();
		String componentDirectory = resultsPath + "/" + cname;
		( new File( componentDirectory ) ).mkdir();
		
		String impl = cname + "_impl";

		openFile( componentDirectory, impl, HPP_EXTENSION );
		
		writeCppHeaderPrelude( component.getModule().getName() );
		
		out_.write( "#include \"microccm_" + cname + "_executor." + HPP_EXTENSION + "\"\n" );
		out_.write( "#include \"microccm_" + cname + "_CORBA." + HPP_EXTENSION + "\"\n" );
		out_.write( "\n" );
		
		out_.write( "class CCM_" + impl + "\n" );
		out_.write( "    : public CCM_" + cname + "\n" );
		out_.write( "    , public CCM_" + cname + "_SessionComponent_impl\n" );
		out_.write( "{\n" );
		out_.write( "public:\n" );
		out_.write( "    CCM_" + impl + "();\n" );
		out_.write( "\n" );
		out_.write( "    void configuration_complete()\n" );
		out_.write( "        throw( Components::InvalidConfiguration ) {}\n" );
		out_.write( "\n" );
		out_.write( "    void shutdown() {}\n" );
		out_.write( "\n" );

		for( Port p : component.getPorts() ) {
			TokenType t = p.getTokenType();
			if( t instanceof StreamType ) {
				// StreamType
				StreamType s = (StreamType) t;
				if( p instanceof ProducerPort ) {
					// source
					out_.write( "    // source " + s.getName() + " "
							  + p.getName() + ";\n" );
				}
				else {
					// sink
					out_.write( "    // sink " + s.getName() + " "
							  + p.getName() + ";\n" );
					out_.write( "public:\n" );
					out_.write( "    void new_" + p.getName() + "_value( "
							  + getCorbaType( s.getType().getName() ) +" value );\n" );
					//out_.write( "    " + STI_MODULE_NAME + "::CCM_"
					out_.write( "    " + "CCM_"
							  + getStreamType( s.getType().getName() ) + "_ptr "
							  + "get_" + p.getName() + "() {\n" );
					out_.write( "        return &" + p.getName() + "_provider_;\n" );
					out_.write( "    }\n" );
					out_.write( "\n" );
					out_.write( "private:\n" );
					out_.write( "    struct Provides_" + p.getName() + "\n" );
					//out_.write( "        : " + STI_MODULE_NAME + "::CCM_"
					out_.write( "        : " + "CCM_"
							  + getStreamType( s.getType().getName() ) + "\n" );
					out_.write( "    {\n" );
					out_.write( "        Provides_" + p.getName()
							  + "( CCM_" + impl + " * c )\n" );
					out_.write( "            : component_( c ) {}\n" );
					out_.write( "\n" );
					out_.write( "        void put( "
							  + getCorbaType( s.getType().getName() )
							  + " value );\n" );
					out_.write( "\n" );
					out_.write( "        CCM_" + impl + " * component_;\n" );
					out_.write( "    };\n" );
					out_.write( "\n" );
					out_.write( "    Provides_" + p.getName() + " "
							  + p.getName() + "_provider_;\n" );
					out_.write( "\n" );
				}
			}
			else {
				// EventType
				if( p instanceof ProducerPort ) {
					// publishes
					out_.write( "    // publishes " + t.getName() + " "
							  + p.getName() + ";\n" );
					out_.write( "\n" );
				}
				else {
					// consumes
					EventType e = (EventType) p.getTokenType();
					out_.write( "    // GENERATED FROM : consumes " + e.getName() + " "
							  + p.getName() + ";\n" );
					out_.write( "    void push_" + p.getName() + "( "
							  + e.getModule().getName() + "::"
							  + e.getName() + " * evt );\n" );
					out_.write( "\n" );
				}
			}
		}
		
		out_.write( "\n" );
		out_.write( "};\n" );
		out_.write( "\n" );

		String home = cname + "Home";
		String home_impl = home + "_impl";

		out_.write( "class " + home_impl + "\n" );
		out_.write( "    : public CCM_" + home + "\n" );
		out_.write( "{\n" );
		out_.write( "public:\n" );
		out_.write( "    Components::EnterpriseComponent_ptr create()\n" );
		out_.write( "        throw( Components::CCMException );\n" );
		out_.write( "};\n" );
		out_.write( "\n" );
		
		out_.write( "extern \"C\"\n" );
		out_.write( "{\n" );
		out_.write( "    Components::HomeExecutorBase * create_" + cname + "_home();\n" );
		out_.write( "}\n" );
		out_.write( "\n" );

		writeCppHeaderPostlude();
		
		closeFile();
		
		openFile( componentDirectory, impl, CPP_EXTENSION );
		
		writeCppCopyrightComment();
		
		out_.write( "#include \"" + impl + "." + HPP_EXTENSION + "\"\n" );
		out_.write( "\n" );

		if( "esterel".equalsIgnoreCase( component.getTool() )) {
			generateEsterelGlueFile( component );
		}
		else if( "lustre".equalsIgnoreCase( component.getTool() )) {
			generateLustreGlueFile( component );
		}
		else if( "simulink".equalsIgnoreCase( component.getTool() )) {
			generateSimulinkGlueFile( component );
		}
		else {
			// TODO
		}
		
		out_.write( "//---------------------------------------------------------------------\n\n" );

		out_.write( "Components::EnterpriseComponent_ptr\n" );
		out_.write( home_impl + "::create() throw( Components::CCMException )\n" );
		out_.write( "{\n" );
		out_.write( "    return new CCM_" + impl + "();\n" );
		out_.write( "}\n" );
		out_.write( "\n" );

		out_.write( "extern \"C\"\n" );
		out_.write( "{\n" );
		out_.write( "    Components::HomeExecutorBase * create_" + cname + "_home()\n" );
		out_.write( "    {\n" );
		out_.write( "        return new " + home_impl + "();\n" );
		out_.write( "    }\n" );
		out_.write( "}\n" );
		out_.write( "\n" );

		closeFile();
	}
	
	private void generateEsterelGlueFile( ComponentType component ) throws IOException {
		String cname = component.getName();
		String impl = cname + "_impl";

		// The current context must be accessible to Esterel output actions
		out_.write( "static ::CCM_" + cname + "_Context_ptr current_component_context = 0;\n" );
		out_.write( "\n" );
		
		// Output actions must be declared before including the generated c file 
		for( ProducerPort p : helper_.getComponentEventProducerPorts( component )) {
			// consumes
			EventType e = (EventType) p.getTokenType();
			out_.write( "void " + cname + "_O_"
					  + p.getName() + "()\n" );
			out_.write( "{\n" );
			out_.write( "#ifdef TRACE_ADLV\n" );
			out_.write( "    std::clog << \"TRACE_ADLV "
					  + ": component : " + component.getName()
					  + ", publishes : " + p.getName()
					  + ", event sent\\n\";\n" );
			out_.write( "#endif\n" );
			out_.write( "\n" );
			out_.write( "    "
					  + e.getModule().getName() + "::"
					  + e.getName() + "_var evt = new OBV_"
					  + e.getModule().getName() + "::"
					  + e.getName() + "();\n" );
			out_.write( "    current_component_context->push_"
					  + p.getName() + "( evt.in() );\n" );
			out_.write( "}\n" );
			out_.write( "\n" );
		}

		//out_.write( "extern \"C\" {\n" );
		out_.write( "#include \"esterel/" + cname + ".c\"\n" );
		//out_.write( "}\n" );
		out_.write( "\n" );
		
		out_.write( "CCM_" + impl + "::CCM_" + impl + "()\n" );
		out_.write( "{\n" );
		out_.write( "    " + cname + "_reset();\n" );
		out_.write( "}\n" );
		out_.write( "\n" );

		for( ConsumerPort p : helper_.getComponentEventConsumerPorts( component )) {
			// consumes
			EventType e = (EventType) p.getTokenType();
			out_.write( "void CCM_" + impl + "::push_" + p.getName() + "( "
					  + e.getModule().getName() + "::"
					  + e.getName() + " * evt )\n" );
			out_.write( "{\n" );
			out_.write( "#ifdef TRACE_ADLV\n" );
			out_.write( "    std::clog << \"TRACE_ADLV "
					  + ": component : " + component.getName()
					  + ", consumes : " + p.getName()
					  + ", event received\\n\";\n" );
			out_.write( "#endif\n" );
			out_.write( "\n" );
			out_.write( "    " + cname + "_I_"
					  + p.getName() + "();\n" );
			out_.write( "    current_component_context = component_context_;\n" );
			out_.write( "    " + cname + "();\n" );
			out_.write( "}\n" );
			out_.write( "\n" );
		}
	}
	
	private void generateLustreGlueFile( ComponentType component ) throws IOException {
		// TODO
	}
	
	private void generateSimulinkGlueFile( ComponentType component ) throws IOException {
		String cname = component.getName();
		String impl = cname + "_impl";
		
		out_.write( "#include \"simulink/" + cname + ".h\"\n" );
		out_.write( "\n" );
		
		// They are not declared in a header file ??
		out_.write( "extern \"C\" RT_MODEL_" + cname + " * " + cname + "( void );\n" );
		out_.write( "extern \"C\" void MdlOutputs( int_T tid );\n" );
		out_.write( "\n" );

		out_.write( "CCM_" + impl + "::CCM_" + impl + "()\n" );
		boolean first_port = true;
		
		// Constructor
		for( ConsumerPort p : helper_.getComponentStreamConsumerPorts( component )) {
			// sink
			//StreamType s = (StreamType) p.getTokenType();
			if( first_port ) {
				out_.write( "    : " );
				first_port = false;
			}
			else {
				out_.write( "    , " );
			}
			out_.write( p.getName() + "_provider_( this )\n" );
		}
		out_.write( "{\n" );
		out_.write( "    " + cname + "();\n" );
		out_.write( "}\n" );
		out_.write( "\n" );
		
		for( ConsumerPort p : helper_.getComponentStreamConsumerPorts( component )) {
			// sink
			StreamType s = (StreamType) p.getTokenType();
			out_.write( "void CCM_" + impl + "::new_"
					  + p.getName() + "_value( "
					  + getCorbaType( s.getType().getName() )
					  + " " + p.getName() + " )\n" );
			out_.write( "{\n" );
			
			out_.write( "    " + cname+ "_U." + p.getName() + " = " + p.getName() + ";\n" );
			out_.write( "    // Ask the Simulink module to compute its outputs\n" );
			out_.write( "    MdlOutputs( 1 );\n" );
			
			out_.write( "    // We use the current flow of control to update our consumers\n" );
			for( ProducerPort p2 : helper_.getComponentStreamProducerPorts( component )) {
				out_.write( "    {\n" );
				StreamType s2 = (StreamType) p2.getTokenType();
				//out_.write( "        " + STI_MODULE_NAME + "::"
				out_.write( "        "
						  + getStreamType( s2.getType().getName() )
						  + "_ptr cl = component_context_->get_connection_"
						  + p2.getName()
						  + "();\n" );

				out_.write( "        if( cl ) {\n" );
				out_.write( "#ifdef TRACE_ADLV\n" );
				out_.write( "            std::clog << \"TRACE_ADLV "
						  + ": component : " + component.getName()
						  + ", uses : " + p2.getName()
						  + ", value sent : \" << " + cname+ "_Y." + p2.getName() + " << \"\\n\";\n" );
				out_.write( "#endif\n" );
				out_.write( "\n" );

				out_.write( "            cl->put( " + cname+ "_Y." + p2.getName() + " );\n" );
				out_.write( "        }\n" );
				out_.write( "    }\n" );
			}
			
			out_.write( "}\n" );
			out_.write( "\n" );
		}

		for( ConsumerPort p : helper_.getComponentStreamConsumerPorts( component )) {
			// sink
			StreamType s = (StreamType) p.getTokenType();
			out_.write( "void CCM_" + impl + "::Provides_" + p.getName()
					  + "::put( "
					  + getCorbaType( s.getType().getName() )
					  + " value )\n" );
			out_.write( "{\n" );
			out_.write( "#ifdef TRACE_ADLV\n" );
			out_.write( "    std::clog << \"TRACE_ADLV "
					  + ": component : " + component.getName()
					  + ", sink : " + p.getName()
					  + ", value received : \" << value << \"\\n\";\n" );
			out_.write( "#endif\n" );
			out_.write( "\n" );
			out_.write( "    component_->new_" + p.getName() + "_value( value );\n" );
			out_.write( "}\n" );
			out_.write( "\n" );
		}
		out_.write( "\n" );
		
	}
	
	private EmfAdlvHelper helper_;
}
