/*******************************************************************************
 * Copyright (c) 2007 Supelec.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This file is part of the ADLV tool
 * Version 1.0 (December 5, 2007)
 *
 * Contributors:
 *     Computer Science Department, Supelec : initial implementation
 * Contacts:
 *     Dominique.Marcadet@supelec.fr
 *     Frederic.Boulanger@supelec.fr
 *     Christophe.Jacquet@supelec.fr
 *******************************************************************************/
package fr.supelec.adlv.generator;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;

import fr.supelec.adlv.*;

import fr.supelec.adlv.parser.EmfAdlvHelper;

public class MpcGenerator extends AdlvGenerator {

	MpcGenerator( EmfAdlvHelper helper ) {
		helper_ = helper;
	}
	
	void generate( String resultsPath ) throws IOException {
		
		generateMwcFile( resultsPath );
		
		HashSet< ComponentType > components = new HashSet< ComponentType >();
		for( Component c : helper_.getApplication().getComponents() ) {
			components.add( c.getType() );
		}
		for( ComponentType t : components ) {
			generateMpcFileForComponent( t, resultsPath );
		}
		
		generateMpcFileForDynamicRouting( resultsPath );
		generateMpcFileForEnvironment( resultsPath );
	}
	
	private void generateMwcFile( String resultsPath ) throws IOException {
		
		openFile( resultsPath, helper_.getApplication().getName(), MAKE_WORKSPACE_EXTENSION );
		
		writeMpcCopyrightComment();
		
		out_.write( "\n" );
		out_.write( "workspace {\n" );
		out_.write( "\n" );
		
		out_.write( "\t./" + STI_MODULE_DIRECTORY
				  + "/" + STREAM_TYPE_INTERFACE_FILENAME
				  + "." + MAKE_PROJECT_EXTENSION
				  + "\n" );

		HashSet< ComponentType > components = new HashSet< ComponentType >();
		for( Component c : helper_.getApplication().getComponents() ) {
			components.add( c.getType() );
		}
		for( ComponentType t : components ) {
			out_.write( "\t./" + t.getName() + "/"
				      + t.getName() + "." + MAKE_PROJECT_EXTENSION + "\n" );
		}

		out_.write( "\t./" + DYNAMIC_ROUTING_NAME
				  + "/" + DYNAMIC_ROUTING_NAME
				  + "." + MAKE_PROJECT_EXTENSION + "\n" );
		
		out_.write( "\t./" + ENVIRONMENT_NAME
				  + "/" + ENVIRONMENT_NAME
				  + "." + MAKE_PROJECT_EXTENSION + "\n" );
		out_.write( "\n" );
		out_.write( "}\n" );
		out_.write( "\n" );
		
		closeFile();
	}
	
	private void generateMpcFileCommonPart( String cname, String tool, Collection< Port > ports, boolean simulation ) throws IOException {
		String impl = cname + IMPL_SUFFIX;

		writeMpcCopyrightComment();
		
		out_.write( "\n" );
		out_.write( "project( " + impl + " ) : " + MPC_SUPER_PROJECT + " {\n" );
		out_.write( "\n" );
		
		if( STATIC_DEPLOYMENT ) {
			if( STATIC_LIBRARIES ) {
				out_.write( "    sharedname = \n" );
				out_.write( "    staticname = " + impl + MCCM_TARGET_SUFFIX + "\n" );
			}
			else {
				out_.write( "    sharedname = " + impl + MCCM_TARGET_SUFFIX + "\n" );
				out_.write( "    staticname = \n" );
			}
			// The following define is needed for static deployment with microccm
			out_.write( "    macros += CUSTOM_ENTRYPT=create_servant_for_" + cname + "_home\n" );
		}
		else if ( CORBA_DEPLOYMENT ) {
			if( STATIC_LIBRARIES ) {
				// TODO
			}
			else {
				out_.write( "    sharedname = " + impl + MCCM_TARGET_SUFFIX + "\n" );
				out_.write( "    staticname = \n" );
				out_.write( "    libs += TAO_CosEvent_Skel EventFactoryStubsTAO\n" );
			}
		}

		out_.write( "    libout = " + INSTALL_DIRECTORY + "\n" );
		out_.write( "\n" );

		out_.write( "\n" );
		out_.write( "    IDL3IDL2_Files {\n" );
		out_.write( "        " + MPC_COMMAND_FLAGS + cname + " " + cname + "Home\n" );
		out_.write( "        " + cname + "." + IDL3_EXTENSION + " >> microccm_" + cname + "." + IDL_EXTENSION + " " );

		out_.write( "\n" );
		out_.write( "    }\n" );
		out_.write( "\n" );
		
		out_.write( "\n" );
		out_.write( "    IDL3CPPEXECUTORS_Files {\n" );
		out_.write( "        " + MPC_COMMAND_FLAGS + cname + " " + cname + "Home\n" );
		out_.write( "        " + cname + "." + IDL3_EXTENSION + " >> "
				          + "microccm_" + cname + "_executor." + CPP_EXTENSION + " " 
				          + "microccm_" + cname + "_executor." + HPP_EXTENSION + "\n" );
		out_.write( "    }\n" );
		out_.write( "\n" );
		
		out_.write( "\n" );
		out_.write( "    IDL3CPPCORBA_Files {\n" );
		out_.write( "        " + MPC_COMMAND_FLAGS + cname + " " + cname + "Home gen_desc_file.txt\n" );
		out_.write( "        " + cname + "." + IDL3_EXTENSION + " >> "
				          + "microccm_" + cname + "_CORBA." + CPP_EXTENSION + " " 
				          + "microccm_" + cname + "_CORBA." + HPP_EXTENSION + "\n" );
		out_.write( "    }\n" );
		out_.write( "\n" );
		
		out_.write( "\n" );
		out_.write( "    IDL_Files {\n" );
		out_.write( "        microccm_" + cname + "." + IDL_EXTENSION + "\n" );
		for( Port p : ports ) {
			TokenType t = p.getTokenType();
			if( t instanceof StreamType ) {
				// StreamType
				StreamType s = (StreamType) t;
				out_.write( "        microccm_" + getStreamType( s.getType().getName() ) + "." + IDL_EXTENSION + "\n" );
			}
			else {
				// EventType
				out_.write( "        microccm_" + t.getModule().getName() + "_" + t.getName() + "." + IDL_EXTENSION + "\n" );
			}
		}
		if( simulation ) {
			out_.write( "        microccm_" + "SimulationComponent" + "." + IDL_EXTENSION + "\n" );
		}
		
		out_.write( "    }\n" );
		out_.write( "\n" );
		
		out_.write( "\n" );
		out_.write( "    Source_Files {\n" );
		out_.write( "        microccm_" + cname + "_executor." + CPP_EXTENSION + "\n" );
		out_.write( "        microccm_" + cname + "_CORBA."    + CPP_EXTENSION + "\n" );
		out_.write( "        "          + cname + "_impl."     + CPP_EXTENSION + "\n" );
		if( "simulink".equalsIgnoreCase( tool )) {
			out_.write( "        simulink/"      + cname + "."          + CPP_EXTENSION + "\n" );
			out_.write( "        simulink/"      + cname + "_data."     + CPP_EXTENSION + "\n" );
			out_.write( "        simulink/"      + "rt_nonfinite."      + CPP_EXTENSION + "\n" );
		}
		out_.write( "    }\n" );
		out_.write( "\n" );
		
		out_.write( "\n" );
		out_.write( "    Header_Files {\n" );
		out_.write( "        microccm_" + cname + "_executor." + HPP_EXTENSION + "\n" );
		out_.write( "        microccm_" + cname + "_CORBA."    + HPP_EXTENSION + "\n" );
		out_.write( "        "          + cname + "_impl."     + HPP_EXTENSION + "\n" );
		out_.write( "    }\n" );
		out_.write( "\n" );
		
		out_.write( "\n" );
		out_.write( "}\n" );
		out_.write( "\n" );
	}
	
	private void generateMpcFileForComponent( ComponentType component, String resultsPath ) throws IOException {
		String cname = component.getName();
		String componentDirectory = resultsPath + "/" + cname;
		( new File( componentDirectory ) ).mkdir();

		openFile( componentDirectory, cname, MAKE_PROJECT_EXTENSION );
		
		generateMpcFileCommonPart( component.getName(), component.getTool(), component.getPorts(), false );
		
		closeFile();
	}
	
	private void generateMpcFileForEnvironment( String resultsPath ) throws IOException {
		String componentDirectory = resultsPath + "/" + ENVIRONMENT_NAME;
		( new File( componentDirectory ) ).mkdir();

		openFile( componentDirectory, ENVIRONMENT_NAME, MAKE_PROJECT_EXTENSION );
		
		HashSet< Port > ports = new HashSet< Port >();
		// We also provide the internal consumer ports (those used for dynamic
		// configuration) so that they can be examined
		//ports.addAll( helper_.getApplicationExternalConsumerPorts() );
		ports.addAll( helper_.getApplicationConsumerPorts() );
		ports.addAll( helper_.getApplicationExternalProducerPorts() );
		
		generateMpcFileCommonPart( ENVIRONMENT_NAME, "", ports, true );
		
		out_.write( "\n" );
		out_.write( "project( " + SIMULATION_NAME + " ) : " + MPC_SUPER_CLIENT + " {\n" );
		out_.write( "\n" );
		out_.write( "    after +=" );
		HashSet< ComponentType > components = new HashSet< ComponentType >();
		for( Component c : helper_.getApplication().getComponents() ) {
			components.add( c.getType() );
		}
		for( ComponentType t : components ) {
			out_.write( " " + t.getName() + IMPL_SUFFIX );
		}
		out_.write( " " + DYNAMIC_ROUTING_NAME + IMPL_SUFFIX
				  + " " + ENVIRONMENT_NAME     + IMPL_SUFFIX );
		out_.write( " " + BOOLEAN_DUPLICATOR   + IMPL_SUFFIX
				  + " " +  DOUBLE_DUPLICATOR   + IMPL_SUFFIX
				  + " " +    LONG_DUPLICATOR   + IMPL_SUFFIX );
		out_.write( "\n" );
		
		out_.write( "    exename = " + SIMULATION_NAME + "\n" );
		
		out_.write( "    libs += ORBHelper_$(MCCM_TARGET) service_integration_$(MCCM_TARGET)" );
		for( ComponentType t : components ) {
			out_.write( " " + t.getName() + "Impl_$(MCCM_TARGET)" );
		}
		out_.write( " " + DYNAMIC_ROUTING_NAME + IMPL_SUFFIX + MCCM_TARGET_SUFFIX );
		out_.write( " " + ENVIRONMENT_NAME     + IMPL_SUFFIX + MCCM_TARGET_SUFFIX );
		out_.write( " " + BOOLEAN_DUPLICATOR   + IMPL_SUFFIX + MCCM_TARGET_SUFFIX );
		out_.write( " " +  DOUBLE_DUPLICATOR   + IMPL_SUFFIX + MCCM_TARGET_SUFFIX );
		out_.write( " " +    LONG_DUPLICATOR   + IMPL_SUFFIX + MCCM_TARGET_SUFFIX );
		out_.write( " TAO_CosEvent_Skel EventFactoryStubsTAO" );
		out_.write( "\n" );

		out_.write( "    libpaths += " + INSTALL_DIRECTORY + "\n" );
		out_.write( "    install = "   + INSTALL_DIRECTORY + "\n" );

		out_.write( "    includes += $(MICROCCM_DIR)/tools/orbhelper\n" );
		out_.write( "    IDL_Files {\n" );
		out_.write( "    }\n" );

		out_.write( "    Source_Files {\n" );
		if( STATIC_DEPLOYMENT ) {
			out_.write( "        runtime.cpp\n" );
		}
		out_.write( "    }\n" );

		out_.write( "\n" );
		out_.write( "}\n" );
		out_.write( "\n" );
		
		closeFile();
	}
	
	private void generateMpcFileForDynamicRouting( String resultsPath ) throws IOException {
		String componentDirectory = resultsPath + "/" + DYNAMIC_ROUTING_NAME;
		( new File( componentDirectory ) ).mkdir();

		openFile( componentDirectory, DYNAMIC_ROUTING_NAME, MAKE_PROJECT_EXTENSION );
		
		HashSet< Port > ports = new HashSet< Port >();
		for( ConsumerPort p : helper_.getApplicationConsumerPorts() ) {
			// Looking for event consumers
			TokenType t = p.getTokenType();
			if( t instanceof StreamType ) continue;
			
			// that are used for dynamic configuration
			if( p.getSteps().isEmpty() ) continue;

			ports.add( p );
			
			for( ConfigurationStep step : p.getSteps() ) {
				if( step instanceof ComponentConfigurationStep ) continue;
				
				ports.add((( ConnectionConfigurationStep ) step ).getConnection().getFromPort() );
				ports.add((( ConnectionConfigurationStep ) step ).getConnection().getToPort() );
			}
		}
		
		generateMpcFileCommonPart( DYNAMIC_ROUTING_NAME, "", ports, false );
		
		closeFile();
	}
	
	private EmfAdlvHelper helper_;
	
	private static final String MPC_SUPER_PROJECT = "mccm_comp_project, mccm_orbhelper";
	private static final String MPC_SUPER_CLIENT = "mccm_comp_client, mccm_orbhelper";
	private static final String MPC_COMMAND_FLAGS = "commandflags = -I../Events -I../Supelec ";
}
